"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formatDataModels = exports.formatData = exports.filterByPatternRules = exports.default = void 0;
var _graphin = require("@antv/graphin");
// 模式匹配结果根据条件进行筛选
var filterByPatternRules = function filterByPatternRules(oriGraphData, pattern, matches, directed) {
  var _a, _b, _c, _d;
  var graphNodeTypeMap = {};
  var nodeDataMap = {};
  oriGraphData.nodes.forEach(function (node) {
    graphNodeTypeMap[node.id] = node.label;
    nodeDataMap[node.id] = node;
  });
  var patternEdgesLength = pattern.edges.length;
  var patternNodesLength = pattern.nodes.length;
  var matchesLength = matches.length;
  for (var i = matchesLength - 1; i >= 0; i -= 1) {
    var match = matches[i];
    // 在 match 找每一个 pattern 点的匹配。只要有一个 pattern 节点没有找到匹配，则该 match 不合格
    var matchNodesLength = match.nodes.length;
    var allPatternNodeMatched = true;
    for (var pn = 0; pn < patternNodesLength; pn += 1) {
      var pNode = pattern.nodes[pn];
      var oneNodeMatched = false;
      // 若该 pattern 点没有规则，则不需要寻找。继续匹配下一个 pattern 点
      if (!((_a = pNode.rules) === null || _a === void 0 ? void 0 : _a.length)) continue;
      for (var mn = 0; mn < matchNodesLength; mn += 1) {
        var mNode = match.nodes[mn];
        // 类型的匹配
        if (pNode.label !== mNode.label) continue; // 不匹配，继续下一个 mNode
        // 规则的匹配
        var mProperties = ((_b = nodeDataMap[mNode.id]) === null || _b === void 0 ? void 0 : _b.data) || mNode.data;
        mNode.data = mProperties;
        if (!mProperties && ((_c = pNode.rules) === null || _c === void 0 ? void 0 : _c.length)) continue; // 不匹配，继续下一个 mNode
        var allRuleMatched = true;
        for (var r = 0; r < pNode.rules.length; r += 1) {
          var rule = pNode.rules[r];
          var mPropertyValue = mProperties[rule.name];
          switch (rule.name) {
            case 'id':
              mPropertyValue = mNode.id;
              break;
            case 'label':
              mPropertyValue = mNode.label;
              break;
            default:
              break;
          }
          switch (rule.rule) {
            case '>':
              if (mPropertyValue <= rule.value) allRuleMatched = false;
              break;
            case '<':
              if (mPropertyValue >= rule.value) allRuleMatched = false;
              break;
            case '=':
              if (mPropertyValue !== rule.value) allRuleMatched = false;
              break;
            case '!=':
              if (mPropertyValue === rule.value) allRuleMatched = false;
              break;
            case 'like':
              if (!"".concat(mPropertyValue).includes(rule.value)) allRuleMatched = false;
              break;
            case 'unlike':
              if ("".concat(mPropertyValue).includes(rule.value)) allRuleMatched = false;
              break;
            default:
              break;
          }
          if (!allRuleMatched) break;
        }
        // 该边符合 pNode，即有一个点符合了 pNode，继续找下一条 pNode 的匹配
        if (allRuleMatched) {
          oneNodeMatched = true;
          break;
        }
      }
      // 该 pEdge 没有找到匹配，该 match 不合格
      if (!oneNodeMatched) {
        allPatternNodeMatched = false;
        break;
      }
    }
    // 有一条 pattern 中的节点没有在该 match 中找到匹配，该 match 不合格，删除该 match
    if (!allPatternNodeMatched) {
      matches.splice(i, 1);
      continue;
    }
    // 在 match 找每一个 pattern 边的匹配。只要有一个 pattern 边没有找到匹配，则该 match 不合格
    var allPatternEdgeMatched = true;
    for (var pe = 0; pe < patternEdgesLength; pe += 1) {
      var pEdge = pattern.edges[pe];
      var oneEdgeMatched = false;
      // 若该 pattern 边没有规则，则不需要寻找
      if (!pEdge.rules || !pEdge.rules.length) continue;
      for (var me = 0; me < match.edges.length; me += 1) {
        var mEdge = match.edges[me];
        // 类型和端点的匹配
        if (pEdge.label !== mEdge.label) continue;
        var directedMatched = pEdge.sourceNodeType === graphNodeTypeMap[mEdge.source] && pEdge.targetNodeType === graphNodeTypeMap[mEdge.target];
        var undirectedMatched = directedMatched || pEdge.sourceNodeType === graphNodeTypeMap[mEdge.target] && pEdge.targetNodeType === graphNodeTypeMap[mEdge.source];
        if (directed && !directedMatched) continue;
        if (!directed && !undirectedMatched) continue;
        // 规则的匹配
        var _mProperties = mEdge === null || mEdge === void 0 ? void 0 : mEdge.data;
        if (!_mProperties && ((_d = pEdge.rules) === null || _d === void 0 ? void 0 : _d.length)) continue;
        var _allRuleMatched = true;
        for (var _r = 0; _r < pEdge.rules.length; _r += 1) {
          var _rule = pEdge.rules[_r];
          var _mPropertyValue = _mProperties[_rule.name];
          switch (_rule.rule) {
            case '>':
              if (_mPropertyValue <= _rule.value) _allRuleMatched = false;
              break;
            case '<':
              if (_mPropertyValue >= _rule.value) _allRuleMatched = false;
              break;
            case '=':
              if (_mPropertyValue !== _rule.value) _allRuleMatched = false;
              break;
            case '!=':
              if (_mPropertyValue === _rule.value) _allRuleMatched = false;
              break;
            case 'like':
              if (!"".concat(_mPropertyValue).includes(_rule.value)) _allRuleMatched = false;
              break;
            case 'unlike':
              if ("".concat(_mPropertyValue).includes(_rule.value)) _allRuleMatched = false;
              break;
            default:
              break;
          }
          if (!_allRuleMatched) break;
        }
        // 该边符合 pEdge，即有一条边符合了 pEdge，继续找下一条 pEdge 的匹配
        if (_allRuleMatched) {
          oneEdgeMatched = true;
          break;
        }
      }
      // 该 pEdge 没有找到匹配，该 match 不合格
      if (!oneEdgeMatched) {
        allPatternEdgeMatched = false;
        break;
      }
    }
    // 有一条 pattern 中的边没有在该 match 中找到匹配，该 match 不合格，删除该 match
    if (!allPatternEdgeMatched) matches.splice(i, 1);
  }
};
exports.filterByPatternRules = filterByPatternRules;
var formatDataModels = function formatDataModels(dataType, models, config) {
  var schemaEdgeMap = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  return (models === null || models === void 0 ? void 0 : models.map(function (model) {
    var _a;
    var type = model["".concat(dataType, "Type")];
    var modelConfig = {};
    if (type) {
      modelConfig = config["".concat(dataType, "s")].find(function (config) {
        return config.groupName === "".concat(type.toUpperCase(), " TYPE");
      }) || modelConfig;
    }
    var _ref = modelConfig.props || {},
      _ref$color = _ref.color,
      color = _ref$color === void 0 ? '#ccc' : _ref$color,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? dataType === 'node' ? 30 : 1 : _ref$size;
    var res = Object.assign(Object.assign({}, model), {
      style: Object.assign(Object.assign({}, model.style), {
        keyshape: {
          size: size,
          fill: dataType === 'node' ? color : undefined,
          stroke: color
        },
        label: typeof model.label === 'string' ? {
          value: model.label
        } : ((_a = model.style) === null || _a === void 0 ? void 0 : _a.label) || model.label
      })
    });
    if (dataType === 'edge') {
      var edgeSchema = schemaEdgeMap[model.edgeType];
      if (edgeSchema) {
        res.sourceNodeType = edgeSchema.sourceNodeType;
        res.targeteEdgeType = edgeSchema.targetNodeType;
      }
    }
    return res;
  })) || [];
};
exports.formatDataModels = formatDataModels;
var formatData = function formatData(graphData, config, schemaEdgeMap) {
  return {
    nodes: formatDataModels('node', graphData.nodes, config),
    edges: _graphin.Utils.processEdges(formatDataModels('edge', graphData.edges, config, schemaEdgeMap))
  };
};
exports.formatData = formatData;
var _default = {
  filterByPatternRules: filterByPatternRules,
  formatDataModels: formatDataModels,
  formatData: formatData
};
exports.default = _default;